// vertex attributes required
attribute vec4 re_Position;
attribute vec3 re_Normal;
attribute vec2 re_TexCoord0;

// vertex shader to fragment shader
varying vec2  vTexCoord;
varying vec4  vDiffuse;
varying vec4  vSpecular;

// scene parameters
uniform vec4 ambient;

// the surface parameters of the mesh
uniform float Kd;
uniform float Ks;
uniform float Ka;
uniform float n_specular;

void main(void) {   
	// Output transformed vertex position:
	gl_Position = re_ModelViewProjectionMatrix * re_Position;

	// Transform vertex position into view and world space:
	vec3 vPos = vec3(re_ModelViewMatrix*re_Position);
	vec3 wPos = vec3(re_ModelMatrix*re_Position);
	
	// Compute the light direction in world space:
	vec3 vLightDiff = re_SpotLights_position[0]-wPos;
	float vLightDistance = length(vLightDiff);
	vec3 vLightDirNorm = vLightDiff / vLightDistance;
	
	vLightDirNorm = vec3(re_ViewMatrix * vec4(vLightDirNorm,0));
	vLightDirNorm = normalize(vLightDirNorm);
	
	// Transform normal into view space:
	vec3 vNormal = normalize( re_NormalMatrix * re_Normal);
	
	// Compute reflection vector in view space:
	vec3 vReflect = normalize( 2.0 * dot(vNormal, vLightDirNorm) * vNormal - vLightDirNorm );         

	// Compute view vector (view space):
	vec3 vecView = -normalize(vPos);                

	// Compute ambient contribution: 
	vDiffuse = ambient * Ka;
	vSpecular = vec4(0.0);
	
	// the dot product between vertex normal and light direction (view space)
	float NdotL = max(dot(vNormal,vLightDirNorm),0.0);
	if(NdotL > 0.0) {
		// convert spot light direction into view space
		vec3 vSpotDirNorm = normalize ( vec3( re_ViewMatrix * vec4(-re_SpotLights_direction[0], 0) ) );
		float spotEffect = dot(vSpotDirNorm, vLightDirNorm);
		if (spotEffect > re_SpotLights_attenDir[0].x) {
			// Compute diffuse contribution: 
			vDiffuse += re_SpotLights_color[0] * Kd * NdotL;
			
			// Compute specular term:
			vSpecular = re_SpotLights_color[0] * Ks * pow(NdotL, n_specular);    

			// apply attenuation
			float attenuation = 1.0 / (
				re_SpotLights_attenPos[0].x +         // constant
				re_SpotLights_attenPos[0].y * vLightDistance +     // linear
				re_SpotLights_attenPos[0].z * vLightDistance * vLightDistance); // quadric
			vDiffuse = vDiffuse * attenuation ; 
			}
	}

	vTexCoord = re_TexCoord0;
}
