/****************************************************************************
Copyright 2010 Bilkent University

Licensed under the Apache License, Version 2.0 (the "License"); 
you may not use this file except in compliance with the License. 
You may obtain a copy of the License at 
http://www.apache.org/licenses/LICENSE-2.0 

Unless required by applicable law or agreed to in writing, software 
distributed under the License is distributed on an "AS IS" BASIS, 
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
See the License for the specific language governing permissions and 
limitations under the License. 
****************************************************************************/

/************************************************************************/
/* Node Stuff                                                           */
/************************************************************************/

inline void Camera::setNode(CameraNode& node){
	if(&node == &mNode) return;
	// inform nodes of the update operation
	mNode.mCamera = 0;
	mNode = node;
	mNode.mCamera = this;
	mNode.mBoundingVolume_WS = &mFrustum_Cache;
	mFrustum_Dirty = true;
}
inline CameraNode& Camera::getNode(){
	return mNode;
}
inline const CameraNode& Camera::getNode() const {
	return mNode;
}

inline const Vector3& Camera::getPosition() const {
	return mNode.getTranslation_World();
}
inline const Quaternion& Camera::getRotation() const{
	return mNode.getRotation_World();
}
inline Vector3 Camera::getDirection() const{
	return mNode.getDirection();
}
inline Vector3 Camera::getRight() const{
	return mNode.getRight();
}
inline Vector3 Camera::getUp() const{
	return mNode.getUp();
}

inline const Matrix4& Camera::getViewMatrix() const{
	return mNode.getViewMatrix();
}
inline uchar Camera::getViewCount() const { 
	return 1; 
}
inline void Camera::setActiveView(uchar viewIndex) { 
	return; 
}
inline void Camera::invalidateFrustum(){
	mFrustum_Dirty=true;
}

inline const GeomPlaneBoundedVolume& Camera::getFrustum_WS() const{
	if(mFrustum_Dirty) updateFrustum();
	return mFrustum_Cache;
}

inline const Matrix4& Camera::getProjectionMatrix() const{
	if(mProjectionMatrix_Dirty) updateProjectionMatrix();
	return mProjectionMatrix_Cache;
}

inline void Camera::setAspectRatio(float aspectRatio){
	mAspectRatio = aspectRatio;
	mProjectionMatrix_Dirty = true;
	mFrustum_Dirty = true;
	mNode.setDirtyFlag(SceneNode::Dirty_Bounding_Volume);
}
inline void Camera::setNearDistance(float _near){
	mNearDistance = _near;
	mProjectionMatrix_Dirty = true;
	mFrustum_Dirty = true;
	mNode.setDirtyFlag(SceneNode::Dirty_Bounding_Volume);
}
inline void Camera::setFarDistance(float _far){
	mFarDistance = _far;
	mProjectionMatrix_Dirty = true;
	mFrustum_Dirty = true;
	mNode.setDirtyFlag(SceneNode::Dirty_Bounding_Volume);
}
inline void Camera::setFieldOfView_y(const Angle& angle){
	mFoV_y = angle;
	mProjectionMatrix_Dirty = true;
	mFrustum_Dirty = true;
	mNode.setDirtyFlag(SceneNode::Dirty_Bounding_Volume);
}

inline float Camera::getAspectRatio() const{
	return mAspectRatio;
}
inline float Camera::getNearDistance() const{
	return mNearDistance;
}
inline float Camera::getFarDistance() const{
	return mFarDistance;
}
inline const AngleRadian& Camera::getFieldOfView_y() const{
	return mFoV_y;
}
inline AngleRadian Camera::getFieldOfView_x() const{
	return AngleRadian(std::atan(std::tan(mFoV_y.getRadian()/2.0f)*mAspectRatio)*2);
}

