/****************************************************************************
Copyright 2010 Bilkent University

Licensed under the Apache License, Version 2.0 (the "License"); 
you may not use this file except in compliance with the License. 
You may obtain a copy of the License at 
http://www.apache.org/licenses/LICENSE-2.0 

Unless required by applicable law or agreed to in writing, software 
distributed under the License is distributed on an "AS IS" BASIS, 
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
See the License for the specific language governing permissions and 
limitations under the License. 
****************************************************************************/

//////////////////////////////////////////////////////////////////////////
// Constructors
inline Color_Real::Color_Real() 
	: r(MIN_VAL),g(MIN_VAL),b(MIN_VAL),a(MAX_VAL) { }
inline Color_Real::Color_Real(ChType _r, ChType _g, ChType _b, ChType _a) 
	: r(_r), g(_g), b(_b), a(_a) { }
inline Color_Real::Color_Real(const Color_Real& val) 
	: r(val.r), g(val.g), b(val.b), a(val.a) { }
inline Color_Real::Color_Real(ChType* val){
	r = val[0];
	g = val[1];
	b = val[2];
	a = val[3];
}

//////////////////////////////////////////////////////////////////////////
// Access
inline void Color_Real::set(ChType _r, ChType _g, ChType _b, ChType _a){
	r = _r;
	g = _g;
	b = _b;
	a = _a;
}
inline void Color_Real::set(ChType* val){
	r = val[0];
	g = val[1];
	b = val[2];
	a = val[3];
}
inline Color_Real::ChType* Color_Real::ptr(){
	return &r;
}
inline const Color_Real::ChType* Color_Real::ptr() const{
	return &r;
}

//////////////////////////////////////////////////////////////////////////
// Comparison
inline bool Color_Real::operator==(const Color_Real& rhs) const {
	if( this->r != rhs.r ) return false;
	if( this->g != rhs.g ) return false;
	if( this->b != rhs.b ) return false;
	if( this->a != rhs.a ) return false;
	return true;
}
inline bool Color_Real::operator!=(const Color_Real& rhs) const {
	return !(*this == rhs);
}

//////////////////////////////////////////////////////////////////////////
// Arithmetic Operations
inline Color_Real Color_Real::operator+(const Color_Real& rhs) const{
	return Color_Real( r+rhs.r, g+rhs.g, b+rhs.b, a+rhs.a );
}
inline Color_Real Color_Real::operator-(const Color_Real& rhs) const{
	return Color_Real( r-rhs.r, g-rhs.g, b-rhs.b, a-rhs.a );
}
inline Color_Real Color_Real::operator*(const Color_Real& rhs) const{
	return Color_Real( r*rhs.r, g*rhs.g, b*rhs.b, a*rhs.a );
}
inline Color_Real Color_Real::operator/(const Color_Real& rhs) const{
	return Color_Real( r/rhs.r, g/rhs.g, b/rhs.b, a/rhs.a );
}
inline Color_Real Color_Real::operator*(ChType scalar    ) const{
	return Color_Real( r+scalar, g+scalar, b+scalar, a+scalar );
}
inline Color_Real Color_Real::operator/(ChType scalar    ) const{
	assert(scalar!=ChType(0.0));
	return Color_Real( r/scalar, g/scalar, b/scalar, a/scalar );
}

inline Color_Real& Color_Real::operator+=(const Color_Real& rhs){
	r += rhs.r;
	g += rhs.g;
	b += rhs.b;
	a += rhs.a;
	return *this;
}
inline Color_Real& Color_Real::operator-=(const Color_Real& rhs){
	r -= rhs.r;
	g -= rhs.g;
	b -= rhs.b;
	a -= rhs.a;
	return *this;
}
inline Color_Real& Color_Real::operator*=(const Color_Real& rhs){
	r *= rhs.r;
	g *= rhs.g;
	b *= rhs.b;
	a *= rhs.a;
	return *this;
}
inline Color_Real& Color_Real::operator/=(const Color_Real& rhs){
	r /= rhs.r;
	g /= rhs.g;
	b /= rhs.b;
	a /= rhs.a;
	return *this;
}

//////////////////////////////////////////////////////////////////////////
// Misc.

inline void Color_Real::compress(){
	if(r<0.0)      r = 0.0;
	else if(r>1.0) r = 1.0;

	if(g<0.0)      g = 0.0;
	else if(g>1.0) g = 1.0;

	if(b<0.0)      b = 0.0;
	else if(b>1.0) b = 1.0;

	if(a<0.0)      a = 0.0;
	else if(a>1.0) a = 1.0;
}

inline void Color_Real::invertRGBA(){
	r = MAX_VAL - r;
	g = MAX_VAL - g;
	b = MAX_VAL - b;
	a = MAX_VAL - a;
}
inline void Color_Real::invertRGB(){
	r = MAX_VAL - r;
	g = MAX_VAL - g;
	b = MAX_VAL - b;
}

inline std::string Color_Real::toString() const{
	char toRet[50]; // large enough
	sprintf(toRet,"R:%1.3f G:%1.3f B:%1.3f A:%1.3f",r,g,b,a);
	std::string s(toRet);
	return s;
}

inline std::ostream& operator<<( std::ostream& o, const Color_Real& c ) {
	o << c.toString();
	return o;
}

inline void Color_Real::getMinMax(ChType& _min, ChType& _max) const{
	_max = r;
	if(g>_max) _max = g;
	if(r>_max) _max = r;
	if(b>_max) _max = b;
	_min = r;
	if(g<_min) _min = g;
	if(r<_min) _min = r;
	if(b<_min) _min = b;
}
//////////////////////////////////////////////////////////////////////////
// Constructors
inline Color_Int::Color_Int() 
	: r(MIN_VAL),g(MIN_VAL),b(MIN_VAL),a(MAX_VAL) { }
inline Color_Int::Color_Int(ChType _r, ChType _g, ChType _b, ChType _a) 
	: r(_r), g(_g), b(_b), a(_a) { }
inline Color_Int::Color_Int(const Color_Int& val) 
	: r(val.r), g(val.g), b(val.b), a(val.a) { }
inline Color_Int::Color_Int(ChType* val){
	r = val[0];
	g = val[1];
	b = val[2];
	a = val[3];
}

//////////////////////////////////////////////////////////////////////////
// Component Access
inline void Color_Int::set(ChType _r, ChType _g, ChType _b, ChType _a){
	r = _r;
	g = _g;
	b = _b;
	a = _a;
}
inline void Color_Int::set(ChType* val){
	r = val[0];
	g = val[1];
	b = val[2];
	a = val[3];
}
inline Color_Int::ChType* Color_Int::ptr(){
	return &r;
}
inline const Color_Int::ChType* Color_Int::ptr() const{
	return &r;
}

//////////////////////////////////////////////////////////////////////////
// Comparison
inline bool Color_Int::operator==(const Color_Int& rhs) const {
	if( this->r != rhs.r ) return false;
	if( this->g != rhs.g ) return false;
	if( this->b != rhs.b ) return false;
	if( this->a != rhs.a ) return false;
	return true;
}
inline bool Color_Int::operator!=(const Color_Int& rhs) const {
	return !(*this == rhs);
}


//////////////////////////////////////////////////////////////////////////
// Arithmetic Operations
inline Color_Int Color_Int::operator+(const Color_Int& rhs) const{
	return Color_Int( r+rhs.r, g+rhs.g, b+rhs.b, a+rhs.a );
}
inline Color_Int Color_Int::operator-(const Color_Int& rhs) const{
	return Color_Int( r-rhs.r, g-rhs.g, b-rhs.b, a-rhs.a );
}

inline Color_Int& Color_Int::operator+=(const Color_Int& rhs){
	r += rhs.r;
	g += rhs.g;
	b += rhs.b;
	a += rhs.a;
	return *this;
}
inline Color_Int& Color_Int::operator-=(const Color_Int& rhs){
	r -= rhs.r;
	g -= rhs.g;
	b -= rhs.b;
	a -= rhs.a;
	return *this;
}

//////////////////////////////////////////////////////////////////////////
// Misc.
inline void Color_Int::invertRGB(){
	r = MAX_VAL - r;
	g = MAX_VAL - g;
	b = MAX_VAL - b;
}
inline void Color_Int::invertRGBA(){
	r = MAX_VAL - r;
	g = MAX_VAL - g;
	b = MAX_VAL - b;
	a = MAX_VAL - a;
}
inline std::string Color_Int::toString() const{
	char toRet[50]; // large enough
	sprintf(toRet,"R:%3d G:%3d B:%3d A:%3d",r,g,b,a);
	std::string s(toRet);
	return s;
}
inline std::ostream& operator<<( std::ostream& o, const Color_Int& c ) {
	o << c.toString();
	return o;
}

inline Color_Packed::Color_Packed() 
	: mType(Color_Pack_RGBA), mColor(0x000000FF) { }
inline Color_Packed::Color_Packed(REng::uint val, REng::Color_Pack_Type _type)
	: mType(_type), mColor(val) {}
inline Color_Packed::Color_Packed(const Color_Packed& val)
	: mType(val.mType), mColor(val.mColor) {}

inline Color_Pack_Type Color_Packed::getType() const{
	return mType;
}
inline uint Color_Packed::get() const{
	return mColor;
}
inline void Color_Packed::get(uchar* _comp) const{
	get(_comp[0], _comp[1], _comp[2], _comp[3]);
}
inline bool Color_Packed::operator==(const Color_Packed& rhs) const{
	if(rhs.getType() == mType){
		return (mColor == rhs.get());
	} else {
		Color_Packed tmp(rhs);
		tmp.setType(mType);
		return (mColor == tmp.get());
	}
}
inline bool Color_Packed::operator!=(const Color_Packed& rhs) const{
	return !(*this == rhs);
}
inline void Color_Packed::invertRGBA(){
	mColor = 0xFFFFFFFF - mColor;
}
inline void Color_Packed::invertRGB(){
	switch(mType){
		case Color_Pack_RGBA:
		case Color_Pack_BGRA:
			mColor = (0xFFFFFF00-(mColor&0xFFFFFF00)) + (mColor&0x000000FF);
			return;
		case Color_Pack_ARGB:
		case Color_Pack_ABGR:
			mColor = (0x00FFFFFF-(mColor&0x00FFFFFF)) + (mColor&0xFF000000);
			return;
	}
}
