/****************************************************************************
Copyright 2010 Bilkent University

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
****************************************************************************/
#include "REng/Geom/GeomCylinder.h"

#include "REng/Math.h"

namespace REng{
	GeomCylinder::GeomCylinder() : mHeight(1.0f), mRadius(1.0f){ ; }
	GeomCylinder::GeomCylinder(float h, float r) : mHeight(h), mRadius(r) { ; }
	GeomCylinder::GeomCylinder(float h, float r, const Vector3& pos)
		: GeomVolume(pos),mHeight(h), mRadius(r) { ; }
	void GeomCylinder::setHeight(float h){
		mHeight = h;
	}
	void GeomCylinder::setRadius(float r){
		mRadius = r;
	}
	void GeomCylinder::setPosition(const Vector3& pos){
		mPosition = pos;
	}
	float GeomCylinder::getHeight() const{
		return mHeight;
	}
	float GeomCylinder::getRadius() const{
		return mRadius;
	}
	float GeomCylinder::getVolume() const{
		return Math::PI*mRadius*mRadius*mHeight;
	}


	bool GeomCylinder::canRotate(){
		return false;
	}
	bool GeomCylinder::canScale(){
		return true;
	}
	bool GeomCylinder::canTranslate(){
		return true;
	}

	void GeomCylinder::rotate_World(const Quaternion& qua){
		/*Matrix4 matRot;
		cml::matrix_rotation_quaternion(matRot,qua);
		mPosition = cml::transform_vector(matRot, mPosition);*/
	}
	void GeomCylinder::scale(const Vector3& vec){
		float minVal(vec[0]);
		if(minVal>vec[1]) minVal = vec[1];
		mRadius *= minVal;
		mHeight *= vec[2];
	}
}
