
//////////////////////////////////////////////////////////////////////////
// CONTACT SURFACE
//////////////////////////////////////////////////////////////////////////

inline Contact_Surface::Contact_Surface(dSurfaceParameters& p){
	mSurface = &p;
	mSurface->mode = 0; // no flags set
	setMU(0.); // only mandatory parameter
}
inline dSurfaceParameters& Contact_Surface::_getODE(){
	return *mSurface;
}
inline void Contact_Surface::setMU(Real mu){
	mSurface->mu = mu;
}
inline void Contact_Surface::setMUInf(){
	mSurface->mu = dInfinity;
}
inline Real Contact_Surface::getMU() const{
	return mSurface->mu;
}
inline void Contact_Surface::set(Member m, Real p1, Real p2){
	switch(m){
		case Member_MU2:
			mSurface->mu2 = p1;
			mSurface->mode |= dContactMu2; break;
		case Member_Bounce:
			mSurface->bounce = p1;
			mSurface->bounce_vel = p2;
			mSurface->mode |= dContactBounce; break;
		case Member_SoftERP:
			mSurface->soft_erp = p1;
			mSurface->mode |= dContactSoftERP; break;
		case Member_SoftCFM:
			mSurface->soft_cfm = p1;
			mSurface->mode |= dContactSoftCFM; break;
		case Member_MotionFDir1:
			mSurface->motion1 = p1;
			mSurface->mode |= dContactMotion1; break;
		case Member_MotionFDir2:
			mSurface->motion2 = p1;
			mSurface->mode |= dContactMotion2; break;
		case Member_MotionFNorm:
			mSurface->motionN = p1;
			mSurface->mode |= dContactMotionN; break;
		case Member_FDS_FDir1:
			mSurface->slip1 = p1;
			mSurface->mode |= dContactSlip1; break;
		case Member_FDS_FDir2:
			mSurface->slip1 = p2;
			mSurface->mode |= dContactSlip2; break;
		case Member_FrictPyramid_FDir1:
			mSurface->mode |= dContactApprox1_1; break;
		case Member_FrictPyramid_FDir2:
			mSurface->mode |= dContactApprox1_2; break;
		case Member_FrictPyramid_Both:
			mSurface->mode |= dContactApprox1; break;
		case Member_FDir1: // TODO
			break;
		default: // not possible
			break;
	}
}
inline void Contact_Surface::reset(Member m){
	switch(m){
		case Member_MU2:
			mSurface->mode &= ~dContactMu2; break;
		case Member_Bounce:
			mSurface->mode &= ~dContactBounce; break;
		case Member_SoftERP:
			mSurface->mode &= ~dContactSoftERP; break;
		case Member_SoftCFM:
			mSurface->mode &= ~dContactSoftCFM; break;
		case Member_MotionFDir1:
			mSurface->mode &= ~dContactMotion1; break;
		case Member_MotionFDir2:
			mSurface->mode &= ~dContactMotion2; break;
		case Member_MotionFNorm:
			mSurface->mode &= ~dContactMotionN; break;
		case Member_FDS_FDir1:
			mSurface->mode &= ~dContactSlip1; break;
		case Member_FDS_FDir2:
			mSurface->mode &= ~dContactSlip2; break;
		case Member_FrictPyramid_FDir1:
			mSurface->mode &= ~dContactApprox1_1; break;
		case Member_FrictPyramid_FDir2:
			mSurface->mode &= ~dContactApprox1_2; break;
		case Member_FrictPyramid_Both:
			mSurface->mode &= ~dContactApprox1; break;
		case Member_FDir1: // TODO
			break;
		default: // not possible
			break;
	}
}
inline bool Contact_Surface::get(Member m, Real& p1, Real& p2){
	switch(m){
		case Member_MU2:
//			if(mSurface->mode & dContactMu2) 
				p1 = mSurface->mu2;
			return mSurface->mode & dContactMu2;
		case Member_Bounce:
//			if(mSurface->mode & dContactBounce) 
				p1 = mSurface->bounce;
//			if(mSurface->mode & dContactBounce) 
				p2 = mSurface->bounce_vel;
			return (mSurface->mode & dContactBounce)!=0;
		case Member_SoftERP:
//			if(mSurface->mode & dContactSoftERP) 
				p1 = mSurface->soft_erp;
			return (mSurface->mode & dContactSoftERP)!=0;
		case Member_SoftCFM:
//			if(mSurface->mode & dContactSoftCFM) 
				p1 = mSurface->soft_cfm;
			return (mSurface->mode & dContactSoftCFM)!=0; 
		case Member_MotionFDir1:
//			if(mSurface->mode & dContactMotion1) 
				p1 = mSurface->motion1;
			return (mSurface->mode & dContactMotion1)!=0; 
		case Member_MotionFDir2:
//			if(mSurface->mode & dContactMotion1) 
				p1 = mSurface->motion2;
			return (mSurface->mode & dContactMotion2)!=0;
		case Member_MotionFNorm:
//			if(mSurface->mode & dContactMotionN) 
				p1 = mSurface->motionN;
			return (mSurface->mode & dContactMotionN)!=0;
		case Member_FDS_FDir1:
//			if(mSurface->mode & dContactSlip1) 
				p1 = mSurface->slip1;
			return (mSurface->mode & dContactSlip1)!=0;
		case Member_FDS_FDir2:
//			if(mSurface->mode & dContactSlip2) 
				p1 = mSurface->slip2;
			return (mSurface->mode & dContactSlip2)!=0;
		case Member_FrictPyramid_FDir1:
			return (mSurface->mode & dContactApprox1_1)!=0; 
		case Member_FrictPyramid_FDir2:
			return (mSurface->mode & dContactApprox1_2)!=0;
		case Member_FrictPyramid_Both:
			return (mSurface->mode & dContactApprox1)!=0;
		case Member_FDir1: // TODO
			break;
		default: // not possible
			break;
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////
// CONTACT COLLISION
//////////////////////////////////////////////////////////////////////////

inline Contact_Collision::Contact_Collision() : mCollision(0){ }
inline Contact_Collision::Contact_Collision(dContactGeom& c){ _setODE(c); }
inline dContactGeom& Contact_Collision::_getODE(){
	return *mCollision;
}
inline void Contact_Collision::_setODE(dContactGeom& c){
	mCollision = &c;
}
inline void Contact_Collision::setPosition(const REng::Vector3& pos){
	mCollision->pos[0] = WORLD_TO_SIM(pos[0]);
	mCollision->pos[1] = WORLD_TO_SIM(pos[1]);
	mCollision->pos[2] = WORLD_TO_SIM(pos[2]);
	mCollision->pos[3] = 0; // filler
}
inline REng::Vector3 Contact_Collision::getPosition() const{
	return REng::Vector3(
		SIM_TO_WORLD(mCollision->pos[0]),
		SIM_TO_WORLD(mCollision->pos[1]),
		SIM_TO_WORLD(mCollision->pos[2])
	);
}

inline void Contact_Collision::setNormal(const REng::Vector3& normal){
	mCollision->normal[0] = normal[0];
	mCollision->normal[1] = normal[1];
	mCollision->normal[2] = normal[2];
}
inline REng::Vector3 Contact_Collision::getNormal() const{
	return REng::Vector3( mCollision->normal[0], mCollision->normal[1], mCollision->normal[2] );
}

inline void Contact_Collision::setDepth(Real depth){
	mCollision->depth = WORLD_TO_SIM(depth);
}
inline Real Contact_Collision::getDepth() const{
	return SIM_TO_WORLD(mCollision->depth);
}

inline Geom* Contact_Collision::getGeom1(){
	return (Geom*)dGeomGetData(mCollision->g1);
}
inline Geom* Contact_Collision::getGeom2(){
	return (Geom*)dGeomGetData(mCollision->g2);
}
inline void Contact_Collision::setGeom1(Geom* g){
	if(g==0) {mCollision->g1 = 0; return;}
	mCollision->g1 = g->_getID();
}
inline void Contact_Collision::setGeom2(Geom* g){
	if(g==0) {mCollision->g2 = 0; return;}
	mCollision->g2 = g->_getID();
}

//////////////////////////////////////////////////////////////////////////
// CONTACT ITSELF
//////////////////////////////////////////////////////////////////////////

inline Contact::Contact(dContact& c){
	contact = &c;
}
inline dContact& Contact::_getODE(){
	return *contact;
}
inline Contact_Collision Contact::collision(){
	return Contact_Collision(contact->geom);
}
inline Contact_Surface Contact::surface(){
	return Contact_Surface(contact->surface);
}
inline void Contact::setFDir1(const REng::Vector3& fdir1){
	contact->fdir1[0] = fdir1[0];
	contact->fdir1[1] = fdir1[1];
	contact->fdir1[2] = fdir1[2];
}
inline REng::Vector3 Contact::getFDir1() const{
	return REng::Vector3(contact->fdir1[0],contact->fdir1[1],contact->fdir1[2]);
}


